from sklearn.neighbors import KNeighborsClassifier
from sklearn.metrics import accuracy_score

import numpy as np
from tensorflow import keras

from sklearn import metrics
import matplotlib.pyplot as plt


n_plot = 4
n_start = 100

print("Loading dataset...")

# Model / data parameters
num_classes = 10
input_shape = (28, 28, 1)

# the data, split between train and test sets
(X_train, y_train), (X_test, y_test) = keras.datasets.mnist.load_data()

_, axes = plt.subplots(2, 4)
images_and_labels = list(zip(X_train, y_train))
for ax, (image, label) in zip(axes[0, :], images_and_labels[n_start:n_start+n_plot]):
    ax.set_axis_off()
    ax.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
    ax.set_title('Training: %i' % label)



# To apply a classifier on this data, we need to flatten the image, to
# turn the data in a (samples, feature) matrix:
n_samples = X_train.shape[0]
x_train = X_train.reshape((X_train.shape[0], -1))
x_test = X_test.reshape((X_test.shape[0], -1))


classifier = KNeighborsClassifier(n_neighbors=5)

print("Train model")
classifier.fit(x_train, y_train)


short_pred = 1000
print("Compute %i predictions " % short_pred)

y_pred = classifier.predict(x_test[n_start:n_start+short_pred,:])
print("Done")


print("Accuracy: ", accuracy_score(y_test[n_start:n_start+short_pred], y_pred))


images_and_predictions_and_gt = list(zip(X_test[n_start:n_start+short_pred,:], y_pred, y_test[n_start:n_start+short_pred]))
for ax, (image, prediction, gt) in zip(axes[1, :], images_and_predictions_and_gt[0:0+n_plot]):
    ax.set_axis_off()
    ax.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
    ax.set_title('Pred %i (gt %i)' % (prediction, gt))

print("Classification report for classifier KNN:\n%s\n"
      % (metrics.classification_report(y_test[n_start:n_start+short_pred], y_pred)) )
cm = metrics.confusion_matrix( y_test[n_start:n_start+short_pred], y_pred )
disp = metrics.ConfusionMatrixDisplay(confusion_matrix=cm,  display_labels=["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"])
disp.plot()

disp.figure_.suptitle("Confusion Matrix KNN (28x28, full data base)")
print("Confusion matrix KNN:\n%s" % disp.confusion_matrix)

plt.show()


nb_image = 0
for (image, prediction, gt) in images_and_predictions_and_gt:
    if prediction != gt and nb_image<n_plot:
        plt.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
        plt.title('Pred %i (gt %i)' % (prediction, gt))
        plt.show()
        nb_image = nb_image + 1