import numpy as np
from tensorflow import keras
from tensorflow.keras import layers
from sklearn import datasets, metrics
import matplotlib.pyplot as plt

from keras.backend.tensorflow_backend import set_session
import tensorflow as tf


num_cores = 4
GPU = 0
if GPU:
    num_GPU = 1
    num_CPU = 1
else:
    num_CPU = 1
    num_GPU = 0

# and configure tensorflow and Keras
config = tf.compat.v1.ConfigProto(intra_op_parallelism_threads=num_cores,
                        inter_op_parallelism_threads=num_cores,
                        allow_soft_placement=True,
                        device_count = {'CPU' : num_CPU,
                                        'GPU' : num_GPU}
                       )
config.gpu_options.allow_growth = True  # dynamically grow the memory used on the GPU
config.gpu_options.per_process_gpu_memory_fraction=0.95
#config.log_device_placement = True  # to log device placement (on which device the operation ran)

sess = tf.Session(config=config)
set_session(sess)  # set this TensorFlow session as the default


n_plot = 4
n_start = 100

print("Loading dataset...")

# Model / data parameters
num_classes = 10
input_shape = (28, 28, 1)

# the data, split between train and test sets
(X_train, y_train), (X_test, y_test) = keras.datasets.mnist.load_data()

_, axes = plt.subplots(2, 4)
images_and_labels = list(zip(X_train, y_train))
for ax, (image, label) in zip(axes[0, :], images_and_labels[n_start:n_start+n_plot]):
    ax.set_axis_off()
    ax.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
    ax.set_title('Training: %i' % label)



# Scale images to the [0, 1] range
x_train = X_train.astype("float32") / 255
x_test = X_test.astype("float32") / 255
# Make sure images have shape (28, 28, 1)
x_train = np.expand_dims(x_train, -1)
x_test = np.expand_dims(x_test, -1)
print("x_train shape:", x_train.shape)
print(x_train.shape[0], "train samples")
print(x_test.shape[0], "test samples")


# convert class vectors to binary class matrices
y_train_tf= keras.utils.to_categorical(y_train, num_classes)
y_test_tf = keras.utils.to_categorical(y_test, num_classes)

model = keras.Sequential(
    [
        keras.Input(shape=input_shape),
        layers.Flatten(),
        layers.Dense(1000, activation="relu"),
        layers.Dense(1000, activation="relu"),
        layers.Dropout(0.5),
        layers.Dense(num_classes, activation="softmax"),
    ]
)

model.summary()

batch_size = 128
epochs = 15

model.compile(loss="categorical_crossentropy", optimizer="adam", metrics=["accuracy"])

model.fit(x_train, y_train_tf, batch_size=batch_size, epochs=epochs, validation_split=0.1)

score = model.evaluate(x_test, y_test_tf, verbose=0)
print("Test loss:", score[0])
print("Test accuracy:", score[1])

predicted = model.predict(x_test)
y_pred = np.argmax(predicted, axis=-1)
images_and_predictions_and_gt = list(zip(X_test, y_pred, y_test))
for ax, (image, prediction, gt) in zip(axes[1, :], images_and_predictions_and_gt[n_start:n_start+n_plot]):
    ax.set_axis_off()
    ax.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
    ax.set_title('Pred %i (gt %i)' % (prediction, gt))

print("Classification report for classifier MLP Keras:\n%s\n"
      % (metrics.classification_report(y_test, y_pred)) )
cm = metrics.confusion_matrix( y_test, y_pred )
disp = metrics.ConfusionMatrixDisplay(confusion_matrix=cm,  display_labels=["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"])
disp.plot()

disp.figure_.suptitle("Confusion Matrix MLP (28x28, full data base)")
print("Confusion matrix MLP:\n%s" % disp.confusion_matrix)

plt.show()