#from mnist import MNIST
from sklearn import ensemble
from sklearn.metrics import accuracy_score
from sklearn import datasets, metrics
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt

n_plot = 4
n_start = 100

print("Loading dataset...")
digits = datasets.load_digits()

# The data that we are interested in is made of 8x8 images of digits, let's
# have a look at the first 4 images, stored in the `images` attribute of the
# dataset.  If we were working from image files, we could load them using
# matplotlib.pyplot.imread.  Note that each image must have the same size. For these
# images, we know which digit they represent: it is given in the 'target' of
# the dataset.
_, axes = plt.subplots(2, 4)
images_and_labels = list(zip(digits.images, digits.target))
for ax, (image, label) in zip(axes[0, :], images_and_labels[n_start:n_start+n_plot]):
    ax.set_axis_off()
    ax.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
    ax.set_title('Training: %i' % label)

# To apply a classifier on this data, we need to flatten the image, to
# turn the data in a (samples, feature) matrix:
n_samples = len(digits.images)
data = digits.images.reshape((n_samples, -1))

# Split data into train and test subsets
X_train, X_test, y_train, y_test = train_test_split(
    data, digits.target, test_size=0.5, shuffle=False)


classifier = ensemble.RandomForestClassifier(n_estimators=100)

print("Train model")
classifier.fit(X_train, y_train)

print("Compute predictions")
predicted = classifier.predict(X_test)

print("Accuracy: ", accuracy_score(y_test, predicted))

images_and_predictions_and_gt = list(zip(digits.images[n_samples // 2:], predicted, y_test))
for ax, (image, prediction, gt) in zip(axes[1, :], images_and_predictions_and_gt[n_start:n_start+n_plot]):
    ax.set_axis_off()
    ax.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
    ax.set_title('Pred %i (gt %i)' % (prediction, gt))

print("Classification report for classifier %s:\n%s\n"
      % (classifier, metrics.classification_report(y_test, predicted)))
disp = metrics.plot_confusion_matrix(classifier, X_test, y_test)
disp.figure_.suptitle("Confusion Matrix RF")
print("Confusion matrix RF:\n%s" % disp.confusion_matrix)

plt.show()

nb_image = 0
for (image, prediction, gt) in images_and_predictions_and_gt:
    if prediction != gt and nb_image<n_plot:
        plt.imshow(image, cmap=plt.cm.gray_r, interpolation='nearest')
        plt.title('Pred %i (gt %i)' % (prediction, gt))
        plt.show()
        nb_image = nb_image + 1